import numpy as np
import scipy
from scipy.optimize import leastsq
import matplotlib.pyplot as plt

print 'numpy, scipy & matplotlib: example 4'
print

def fp(p, x):
    # parametric function
    # parameter vector p, len(3)
    # variable vector x, len(N)
    return p[0]/(x**p[1])*np.sin(p[2]*x)

def errf(p, x, y):
    # error function
    return fp(p,x) - y

# Generating noisy data to fit

# Noisy function (used to generate data to fit)
p_real = [1.5, 0.1, 2.]
fn = lambda x: fp(p_real, x)

n = 50
xmin = 0.1
xmax = 5
x = np.linspace(xmin,xmax,n)
y = fn(x) + np.random.random(len(x))*0.12*(fn(x).max()-fn(x).min())

# Initial parameter value
p0 = [3., 1, 4.]

# Fitting
p, success = leastsq(errf, p0, args=(x,y), maxfev=10000)

# Plot
print 'Initial parameters:   ', np.array2string(np.asarray(p0), precision=2)
print 'Estimater parameters: ', np.array2string(np.asarray(p), precision=2)
print 'Real parameters:      ', np.array2string(np.asarray(p_real), precision=2)

X = np.linspace(xmin,xmax,n*5)
plt.plot(x,y,'ro', label='data')
plt.plot(X, fp(p,X), label='lsqfit')
plt.legend(loc='lower left', numpoints=3)

plt.show()
